/*
VDS.Common is licensed under the MIT License

Copyright (c) 2012-2015 Robert Vesse

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software 
is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

using System.Collections.Generic;

namespace VDS.Common.Trees
{
    /// <summary>
    /// An unbalanced binary tree
    /// </summary>
    /// <typeparam name="TKey">Key Type</typeparam>
    /// <typeparam name="TValue">Value Type</typeparam>
    public sealed class UnbalancedBinaryTree<TKey, TValue>
        : BinaryTree<TKey, TValue>
    {
        /// <summary>
        /// Creates a new unbalanced tree
        /// </summary>
        public UnbalancedBinaryTree()
            : base() { }

        /// <summary>
        /// Creates a new unbalanced tree
        /// </summary>
        /// <param name="comparer">Key Comparer</param>
        public UnbalancedBinaryTree(IComparer<TKey> comparer)
            : base(comparer) { }

        /// <summary>
        /// Creates a new node
        /// </summary>
        /// <param name="parent">Parent</param>
        /// <param name="key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        protected sealed override IBinaryTreeNode<TKey, TValue> CreateNode(IBinaryTreeNode<TKey, TValue> parent, TKey key, TValue value)
        {
            return new BinaryTreeNode<TKey, TValue>(parent, key, value);
        }
    }
}
